<?php

namespace Drupal\gifplayer\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\file\Entity\File;

/**
 * Defines a form that configures gifplayer settings.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity type bundle info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityTypeBundleInfo;

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typedConfigManager
   *   The typed config manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The core entity type manager.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle info service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    TypedConfigManagerInterface $typedConfigManager,
    EntityTypeManagerInterface $entity_type_manager,
    EntityTypeBundleInfoInterface $entity_type_bundle_info,
  ) {
    parent::__construct($config_factory, $typedConfigManager);
    $this->entityTypeManager = $entity_type_manager;
    $this->entityTypeBundleInfo = $entity_type_bundle_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'gifplayer_admin_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'gifplayer.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->configFactory->get('gifplayer.settings');

    $form = ['#attributes' => ['enctype' => 'multipart/form-data']];

    $form['library_attach'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Attach GifPlayer library globally.'),
      '#description' => $this->t(
        'Check it to load GifPlayer library globally. By default this is
        attached only to the GifPlayer field and can be used only inside forms
        the field type is used.'
      ),
      '#default_value' => $config->get('library_attach') ?: FALSE,
    ];

    $form['file_upload_details'] = [
      '#markup' => $this->t(
        '<b>File will be used as cover for the GIF player fields.</b>'
      ),
    ];

    $form['cover_png'] = [
      '#type' => 'managed_file',
      '#name' => 'cover_png',
      '#title' => $this->t('Png cover'),
      '#size' => 20,
      '#description' => $this->t(
        'File used as cover on GIF player fields. Leave this empty to use the
        module default file. PNG formats only.'
      ),
      '#default_value' => ($config->get('cover_png') !== NULL) ? [$config->get('cover_png')] : [0],
      '#upload_validators' => ['FileExtension' => ['extensions' => 'png']],
      '#upload_location' => 'public://my_files/',
    ];

    // Add hidden value of default FID of cover PNG if this has been added.
    $form['cpid'] = [
      '#type' => 'hidden',
      '#value' => ($config->get('cover_png') !== NULL) ? [$config->get('cover_png')] : [0],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    $fid = reset($form_state->getValue('cover_png'));

    if (($form_state->getValue('cpid') !== NULL) && !empty($form_state->getValue('cpid'))) {
      $cpid = reset($form_state->getValue('cpid'));
    }
    else {
      $cpid = 0;
    }

    if (($fid === FALSE) & ($cpid > 0)) {
      return TRUE;
    }

    if ($fid === FALSE) {
      $form_state->setErrorByName('cover_png', $this->t('Png cover cannot be empty.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    $fid = reset($form_state->getValue('cover_png'));
    $cpid = reset($values['cpid']);

    /* If $cpid is not empty and $fid is empty, means that the user has
     * deleted the PNG file */

    // The PNG file hasn't changed.
    if ($fid === $cpid) {
      parent::submitForm($form, $form_state);
    }

    if (isset($fid) && is_numeric($fid)) {
      $file = File::load($fid);

      // There is a new file that wants to replace the old file.
      if (($cpid > 0) && ($fid !== $cpid)) {
        $oldfile = File::load($cpid);
        $oldfile->delete();
      }

      // Set file status permanent.
      if ($fid !== 0) {
        $file->setPermanent();
        $file->save();
      }

    }

    // The user has just deleted the PNG file.
    if (($cpid > 0) && ($fid === FALSE)) {
      $fid = 0;
    }

    // Save the updated settings.
    $this->config('gifplayer.settings')
      ->set('library_attach', $values['library_attach'])
      ->set('cover_png', $fid)
      ->save();

    parent::submitForm($form, $form_state);
  }

}
